# parameters
randrepeat <- 100 # number of times the random experiments are performed


# download data
cat("Now loading the expression data\n")
expression = as.matrix(read.table('sce_dutreix.exp',row.names=1)[,c(7:24)])
# We remove sample 7 which is an outlier
expression <- expression[,-7]
ngenes = dim(expression)[1]
nexp = dim(expression)[2]
cat(paste("ok, I found",nexp,"with",ngenes,"genes\n",sep=" "))
nonirradiated = 1:11
irradiated = 12:17

cat("Now loading the opposite Laplacian matrix\n")
lap = as.matrix(read.table('sce.mat',row.names=1))
cat("Now removing the small connected components\n")
# First we remove the small connected components. It seems that they can be identified from the 5th eigenvector (to be checked if the matrix changes)
eigenlap <- eigen(lap)
maincomponent <- eigenlap$vector[,5] < -0.01
lap <- lap[maincomponent,maincomponent]
expression <- expression[maincomponent,]
ngenes = dim(expression)[1]
cat("Ok, there are",ngenes,"genes in the main component of the graph.\n")

cat("Now computing the eigendecomposition of the graph\n")
eigenlap <- eigen(lap)
nulleigenvalues <- eigenlap$values > -1e-10
cat("Ok, there remains",length(seq(ngenes)[nulleigenvalues]),"connected component\n",sep=" ")

# Mean-center and scale to unit variance each gene
cat("Now normalizing expression data")
expression = t(scale(t(expression)))
# Additionally center each array to be sure the constant component disappears in the spectral analysis
expression <- scale(expression,center=TRUE, scale=FALSE)

cat("Now running the first experiment\n")
betarange = seq(from=0,to=1,length=10)

result = matrix(0,nrow=length(betarange))

for (iter in 1:length(betarange)) {
	# compute the new metric matrix
	beta = betarange[iter]
	
	expeigenvalues <- exp(beta*eigenlap$values)
	# remove the components of the zero eigenvalues
	expeigenvalues[nulleigenvalues] <- 0

	distM <- eigenlap$vector %*% diag(expeigenvalues) %*% t(eigenlap$vector)

	# compute the new Gram matrix
	grammatrix = t(expression) %*% distM %*% expression
	# compute the new distance matrix
	distMatrix = diag(grammatrix) %*% matrix(1,ncol=nexp) + matrix(1,nrow=nexp) %*% t(diag(grammatrix)) - 2*grammatrix
	# evaluate the result
	result[iter] = (sum(distMatrix[nonirradiated,nonirradiated]) + sum(distMatrix[irradiated,irradiated])) / sum(distMatrix) 
}

# repeat the same experiment with a random network obtained by keeping the graph structure but randomly shuffling the vertices.
randresult <- matrix(0,nrow=length(betarange),ncol=randrepeat)
for (irepeat in seq(randrepeat)) {
	expressionrand <- expression[sample(ngenes),]
	for (iter in 1:length(betarange)) {
		beta = betarange[iter]
		expeigenvalues <- exp(beta*eigenlap$values)
		expeigenvalues[nulleigenvalues] <- 0
		distM <- eigenlap$vector %*% diag(expeigenvalues) %*% t(eigenlap$vector)
		grammatrix = t(expressionrand) %*% distM %*% expressionrand
		distMatrix = diag(grammatrix) %*% matrix(1,ncol=nexp) + matrix(1,nrow=nexp) %*% t(diag(grammatrix)) - 2*grammatrix
		randresult[iter,irepeat] = (sum(distMatrix[nonirradiated,nonirradiated]) + sum(distMatrix[irradiated,irradiated])) / sum(distMatrix)
	}
}	

cat("Now plotting the result\n")
postscript(file="unsupervised_beta.eps",horizontal=FALSE,width=10,height=10,paper="special") 
plot(betarange,result2,type='b',xlab="Beta",ylab="Loss",col="red",ylim=c(0.435,0.455))
lines(betarange,apply(randresult2,1,mean),lty=1)
lines(betarange,apply(randresult2,1,mean)-apply(randresult2,1,sd),lty=2)
lines(betarange,apply(randresult2,1,mean)+apply(randresult2,1,sd),lty=2)
dev.off()


cat("Now running the second experiment\n")
# another try by keeping only the largest (nonzero) eigenvectors instead of weighting

niter <- 50
iterskip <- 1 # number of eigenvectors to skip (typically the constant ones)
result2 = matrix(0,nrow=niter-iterskip)

expeigenvalues <- numeric(ngenes)
for (iter in (1+iterskip):niter) {
	 expeigenvalues[iter] = 1
	distM <- eigenlap$vector %*% diag(expeigenvalues) %*% t(eigenlap$vector)

	# compute the new Gram matrix
	grammatrix = t(expression) %*% distM %*% expression
	# compute the new distance matrix
	distMatrix = diag(grammatrix) %*% matrix(1,ncol=nexp) + matrix(1,nrow=nexp) %*% t(diag(grammatrix)) - 2*grammatrix
	# evaluate the result
	result2[iter-iterskip] = (sum(distMatrix[nonirradiated,nonirradiated]) + sum(distMatrix[irradiated,irradiated])) / sum(distMatrix) 
}	 

# same with random permutation of genes
randresult2 <- matrix(0,nrow=niter-iterskip,ncol=randrepeat)
for (irepeat in seq(randrepeat)) {
	expressionrand <- expression[sample(ngenes),]
	keepeigenvalues <- numeric(ngenes)
	for (iter in (1+iterskip):niter) {
		keepeigenvalues[iter] = 1
		distM <- eigenlap$vector %*% diag(keepeigenvalues) %*% t(eigenlap$vector)
		grammatrix = t(expressionrand) %*% distM %*% expressionrand
		distMatrix = diag(grammatrix) %*% matrix(1,ncol=nexp) + matrix(1,nrow=nexp) %*% t(diag(grammatrix)) - 2*grammatrix
		randresult2[iter-iterskip,irepeat] = (sum(distMatrix[nonirradiated,nonirradiated]) + sum(distMatrix[irradiated,irradiated])) / sum(distMatrix)
	}
}


cat("Now plotting the result\n")
postscript(file="unsupervised_threshold.eps",horizontal=FALSE,width=10,height=10,paper="special") 
plot(seq(niter-iterskip),result2,type='b',xlab="Beta",ylab="Loss",col="red",ylim=c(0.43,0.58))
lines(seq(niter-iterskip),apply(randresult2,1,mean),lty=1)
lines(seq(niter-iterskip),apply(randresult2,1,mean)-apply(randresult2,1,sd),lty=2)
lines(seq(niter-iterskip),apply(randresult2,1,mean)+apply(randresult2,1,sd),lty=2)
dev.off()

# *------------------------------------------------------------*
# | Here we do some supervised classification of the data.     |
# |                                                            |
# | NEEDED PACKAGE:                                            |
# |    svmpath                                                 |
# *------------------------------------------------------------*

# classes is the vector containing the classes of the different
# samples
classes <- matrix(0,nrow=18);
classes[1:12] <- 1;
classes[13:18] <- -1;

source('svmpathfunctions.R')


# in order to find the looked-for exponential coefficient, we have
# to do some svm test

eigenlap <- eigen(lap)
betarange = seq(5,10,length=100)
result = matrix(0,nrow=length(betarange))
for (iter in 1:length(betarange)) {
	# compute the new metric matrix
	beta = betarange[iter]

	expinvrootlap <- eigenlap$vector %*% diag(exp(beta*(0.5)*eigenlap$values)) %*% t(eigenlap$vector)
	
	# compute the new datatot matrix
	newdata = expinvrootlap %*% expression;
	resultbuffer<- 0;

	result[iter] <- loo.svmpath(t(newdata),classes,lambdas=seq(from=1,to=100,length=100));
}

# We train the SVM where the results are the best
minimum <- which.min(result);
beta <- betarange[minimum[1]];
eigenlap <- eigen(lap)
expinvrootlap <- eigenlap$vector %*% diag(exp(beta*(0.5)*eigenlap$values)) %*% t(eigenlap$vector)
newdata = expinvrootlap %*% expression;
bestsvm <- svmpath(t(newdata), as.numeric(classes));

# We calculate the corresponding separator
separator <- expinvrootlap %*% coef(bestsvm)$beta %*% coef(bestsvm)$beta0;
write.table(separator, "separator.txt",col.names=FALSE, row.names=FALSE);

# We get the genenames
genenames <- sub("sce.", "", names(lap[1,]));
write(genenames, "genenames.txt");

# We do the same for the euclidian distance
euclisvm <- svm(t(expression),classes,type="C-classification", kernel="linear");
eucliseparator <- t(t(euclisvm$coefs) %*% euclisvm$SV);

write.table(eucliseparator, "eucli.txt",col.names=FALSE, row.names=FALSE);

# In order to have a file formated for kegg pathway viewer, 
# execute:
# perl prepare_kegg_file.pl genenames.txt separator.txt