# Useful functions for svmpath package
# Jean-Philippe.Vert@mines.org
# Feb 20th, 2006
#
# the main function is loo.svmpath(x,y,lambdas) that compute the LOO error, where the regularization parameter is optimized over the grid lambda on each training set by internal LOO cross-validation.


require(svmpath)

loo.internal.svmpath <-
function(x, y, lambdas = seq(from = 0, to = 1, by = 0.01), trace = FALSE, ...)
#====
# Compute the curve of leave-one-out (loo) error for any number of regularization parameters (lambdas), and the estimated best lambda (which minimizes the loo error).
#====
{
	nsamples <- dim(x)[1] # number of samples
	looerror <- numeric(length(lambdas))
	loohinge <- numeric(length(lambdas))
	for (i in seq(nsamples)) {
		predictor <- svmpath(x[-i, ], y[-i], trace = trace, ...)
		classprediction <- predict(predictor, x[i, , drop = FALSE], lambdas, type="class")
		looerror <- looerror - (classprediction * y[i] - 1)/2
		scoreprediction <- predict(predictor, x[i, , drop = FALSE], lambdas, type="function")
		loohinge <- loohinge + pmax(1-scoreprediction * y[i],0)
	}
	object <- list(lambdas = lambdas , looerror = looerror, loohinge = loohinge)
	invisible(object)
}


loo.svmpath <-
function(x, y, lambdas = seq(from = 0, to = 1, by = 0.01), lossfunction = 'hinge', trace = FALSE, ...)
#====
# Perform a rigorous leave-one-out (loo) estimation of the accuracy of a SVM. On each training set, the regularization parameter is optimized by internal loo cross-validation to minimize the number of misclassification.
#
# Return the number of misclassification in LOO.
#====
{
	nsamples <- dim(x)[1] # number of samples
	looerror <- 0
	loohinge <- 0
	for (i in seq(nsamples)) {
		internalloo <- loo.internal.svmpath(x[-i, ], y[-i], lambdas = lambdas , trace = FALSE, ...)  # internal LOO loop to select the regularization parameter
		if (lossfunction == 'hinge') {
			losscurve <- internalloo$loohinge
		} else {
			losscurve <- internalloo$looerror
		}
		if (trace) {
			plot(lambdas,losscurve,type='l')
			bestlambdaindex = max(which(losscurve==min(losscurve))) 
			cat("Fold ",i,": lambda = ",lambdas[bestlambdaindex],"\n",sep="")
		}
		predictor <- svmpath(x[-i, ], y[-i], trace = FALSE, ...)
		classprediction <- predict(predictor, x[i, , drop = FALSE], lambdas[bestlambdaindex], type="class")
		scoreprediction <- predict(predictor, x[i, , drop = FALSE], lambdas[bestlambdaindex], type="function")
		looerror <- looerror - (classprediction * y[i] - 1)/2
		loohinge <- loohinge + pmax(1-scoreprediction * y[i],0)
	}
	invisible(list(looerror=looerror,loohinge=loohinge))
}